# coding=utf-8
"""
用于管理缓存的配置数据
使用前必须先调用 init() 。
"""
import copy
import json
import logging
import os

from mootdx.consts import EX_HOSTS, GP_HOSTS, HQ_HOSTS
from mootdx.utils import get_config_path

logger = logging.getLogger(__name__)

__all__ = ['set', 'get', 'copy', 'update', 'settings']

settings = {
    'SERVER': {'HQ': HQ_HOSTS, 'EX': EX_HOSTS, 'GP': GP_HOSTS},
    'BESTIP': {'HQ': '', 'EX': '', 'GP': ''},
    'TDXDIR': 'C:/new_tdx',
}

BASE = os.path.dirname(os.path.dirname(__file__))
CONF = get_config_path('config.json')


def setup():
    """
    将 yaml 里的配置文件导入到 config.py 中
    :return: bool，true 表示数据导入成功。
    """
    global settings

    try:
        options = json.load(open(CONF))
        settings.update(options)
    except Exception:
        logger.error('未找到配置文件 config.json.')

    return True if settings else False


def has(key, value):
    '''
    通过 key 设置某一项值

    :param key:
    :param value:
    :return:
    '''
    return value in settings[key]


def set(key, value):
    '''
    通过 key 设置某一项值
    :param key:
    :param value:
    :return:
    '''
    settings[key] = value


def get(key, default=None):
    '''
    通过 key 获取值
    :param key:
    :param default:
    :return:
    '''
    key = key.split('.')
    cfg = settings.get(key[0])

    if len(key) > 1:
        for x in key[1:]:
            if cfg.get(x):
                cfg = cfg.get(x)
            else:
                cfg = cfg.get(x, default)
                break

    return cfg


def path(key, value=None):
    '''
    通过 key 构建路径
    :param key:
    :param value:
    :return:
    '''
    path = settings.get(key)
    path = os.path.join(BASE, path, value)

    return path


def clone():
    '''
    复制配置
    :return:
    '''
    return copy.deepcopy(settings)


def update(options):
    '''
    全部替换配置
    :param options:
    :return:
    '''
    settings.update(options)



# -*- coding: utf-8 -*-
from mootdx.consts import HQ_HOSTS, EX_HOSTS, GP_HOSTS

__version__ = '0.3.15'

CONFIG = {
    'SERVER': {
        'HQ': HQ_HOSTS,
        'EX': EX_HOSTS,
        'GP': GP_HOSTS
    },
    'BESTIP': {
        'HQ': '',
        'EX': '',
        'GP': ''
    },
    'TDXDIR': 'C:/new_tdx',
}


def bestip():
    return



# -*- coding: utf-8 -*-
import logging
import os
import platform
from struct import *

import pandas as pd
from mootdx.consts import MARKET_SH, MARKET_SZ
from pandas import DataFrame
from tqdm import tqdm

logger = logging.getLogger(__name__)


def get_stock_markets(symbols=None):
    results = []

    assert isinstance(symbols, list), 'stock code need list type'

    if isinstance(symbols, list):
        for symbol in symbols:
            results.append([get_stock_market(symbol, string=False), symbol])

    return results


def get_stock_market(symbol='', string=False):
    """判断股票ID对应的证券市场
    匹配规则
    ['50', '51', '60', '90', '110'] 为 sh
    ['00', '13', '18', '15', '16', '18', '20', '30', '39', '115'] 为 sz
    ['5', '6', '9'] 开头的为 sh， 其余为 sz
    :param string:
    :param symbol:股票ID, 若以 'sz', 'sh' 开头直接返回对应类型，否则使用内置规则判断
    :return 'sh' or 'sz'"""
    assert isinstance(symbol, str), 'stock code need str type'

    market = None

    if symbol.startswith(('sh', 'sz')):
        market = symbol[:2]

    if symbol.startswith(('50', '51', '60', '90', '110', '113', '132', '204')):
        market = 'sh'

    if symbol.startswith(
        ('00',
         '13',
         '18',
         '15',
         '16',
         '18',
         '20',
         '30',
         '39',
         '115',
         '1318')):
        market = 'sz'

    if symbol.startswith(('5', '6', '9', '7')):
        market = 'sh'

    if string is False:
        # return 0 if market == 'sz' else 1
        market = MARKET_SZ if market == 'sz' else MARKET_SH

    return market


def parse_gpcw(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line.split(',') for line in lineiter]


def gpcw(filepath):
    cw_file = open(filepath, 'rb')
    header_size = calcsize("<3h1H3L")
    stock_item_size = calcsize("<6s1c1L")
    data_header = cw_file.read(header_size)
    stock_header = unpack("<3h1H3L", data_header)
    max_count = stock_header[3]

    for idx in range(0, max_count):
        cw_file.seek(header_size + idx * calcsize("<6s1c1L"))
        si = cw_file.read(stock_item_size)
        stock_item = unpack("<6s1c1L", si)
        code = stock_item[0].decode()
        foa = stock_item[2]
        cw_file.seek(foa)
        info_data = cw_file.read(calcsize('<264f'))
        cw_info = unpack('<264f', info_data)

        logger.debug("{}, {}".format(code, str(cw_info)))
        return code, cw_info


def md5sum(downfile):
    '''
    文件的 md5 哈希值

    :param downfile:
    :return:
    '''
    import hashlib
    md5_l = hashlib.md5()

    with open(downfile, mode="rb") as fp:
        by = fp.read()

    md5_l.update(by)
    ret = md5_l.hexdigest()

    return ret


def to_data(v):
    '''
    数值转换为 pd.DataFrame

    :param v: mixed
    :return: pd.DataFrame
    '''
    if isinstance(v, DataFrame):
        return v
    elif isinstance(v, list):
        return pd.DataFrame(data=v) if len(v) else None
    elif isinstance(v, dict):
        return pd.DataFrame(data=[v, ])
    else:
        return pd.DataFrame(data=[{'value': v}])


def to_file(df, filename=None):
    '''
    根据扩展名输出文件

    :param df: pd.DataFrame
    :param filename: 要输出的文件
    :return: bool
    '''
    logger.debug(filename)

    if filename is None or df is None:
        return None

    path_name = os.path.dirname(filename)

    if path_name and not os.path.isdir(path_name):
        os.makedirs(path_name)

    extension = os.path.splitext(filename)
    extension = extension[1] if len(extension) >= 2 else ''

    if extension == '.csv':
        return df.to_csv(filename, encoding='utf-8', index=False)
    elif extension == '.xlsx' or extension == '.xls':
        # openpyxl, xlwt
        return df.to_excel(filename, index=False)
    elif extension == '.h5':
        # tables
        return df.to_hdf(filename, 'df', index=False)
    elif extension == '.json':
        return df.to_json(filename, orient='records')

    return None


class TqdmUpTo(tqdm):
    """Provides `update_to(n)` which uses `tqdm.update(delta_n)`."""

    def update_to(self, downloaded=0, total_size=None):
        """
        b  : int, optional
            Number of blocks transferred so far [default: 1].
        bsize  : int, optional
            Size of each block (in tqdm units) [default: 1].
        tsize  : int, optional
            Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if total_size is not None:
            self.total = total_size

        # self.ascii = True
        self.update(downloaded - self.n)  # will also set self.n = b * bsize


def get_config_path(config='config.json'):
    if platform.system() == 'Windows':
        filename = os.path.join(os.path.expanduser('~'), 'mootdx', config)
    else:
        filename = os.path.join(os.path.expanduser('~'), '.mootdx', config)

    path_name = os.path.dirname(filename)

    if not os.path.isdir(path_name):
        os.makedirs(path_name)

    return filename



# -*- coding: utf-8 -*-
import logging

from pytdx.trade import TdxTradeApi

logger = logging.getLogger(__name__)


class Trader(object):
    '''
    通达信自动交易
    '''
    client_id = None
    endpoint = None
    client = None
    key = None
    iv = None

    def __init__(self, endpoint, key, iv):
        self.client = TdxTradeApi(endpoint=endpoint, enc_key=bytes(key), enc_iv=bytes(iv))

    def config(self, **kwargs):
        self.client = TdxTradeApi(**kwargs)

    def ping(self):
        '''
        服务器连通测试

        :return:
        '''
        return self.client.ping()

    def connect(self):
        '''
        链接服务器
        :return:
        '''
        if self.client is None:
            self.client = TdxTradeApi(endpoint=self.endpoint, enc_key=bytes(self.key), enc_iv=bytes(self.iv))

    # 登入
    def login(self, ip, port, version, yyb_id, account_id, trade_account, jy_passwrod, tx_password):
        '''
        登录服务器
        :param ip:
        :param port:
        :param version:
        :param yyb_id:
        :param account_id:
        :param trade_account:
        :param jy_passwrod:
        :param tx_password:
        :return:
        '''
        result = self.client.logon(ip, port, version, yyb_id, account_id, trade_account, jy_passwrod, tx_password)

        if result.get('success'):
            self.client_id = result.get('data').get('client_id')

        return self.client_id

    # 登出
    def logout(self):
        '''
        注销
        :return:
        '''
        return self.client.logoff(self.client_id)

    # 查询信息
    def query(self, category):
        '''
        查询信息

        :param category:
        :return:
        '''
        self.client.query_data(client_id=self.client_id, category=category)

    # 查询历史信息
    def history(self, category, begin_date, end_date):
        '''
        查询历史信息

        :param category:
        :param begin_date:
        :param end_date:
        :return:
        '''
        return self.client.query_history_data(self.client_id, category, begin_date, end_date)

    # 创建订单
    def order(self, category, price_type, gddm, zqdm, price, quantity):
        '''
        创建订单

        :param category:
        :param price_type:
        :param gddm:
        :param zqdm:
        :param price:
        :param quantity:
        :return:
        '''
        return self.client.send_order(self.client_id, category, price_type, gddm, zqdm, price, quantity)

    # 撤销订单
    def cancel(self, exchange_id, hth):
        '''
        撤销订单

        :param exchange_id:
        :param hth:
        :return:
        '''
        return self.client.cancel_order(self.client_id, exchange_id, hth)

    # 获取行情
    def quote(self, code):
        '''
        获取行情

        :param code:
        :return:
        '''
        return self.client.get_quote(self.client_id, code)

    # 融资融券账户直接还款
    def repay(self, amount):
        '''
        融资融券账户直接还款

        :param amount:
        :return:
        '''
        return self.client.repay(self.client_id, amount)

    # 获取所有正在登录的client账号列表
    def clients(self):
        '''
        获取所有正在登录的client账号列表

        :return:
        '''
        return self.client.get_active_clients()


if __name__ == '__main__':
    client = Trader(endpoint="http://10.11.5.175:10092/api", key="4f1cf3fec4c84c84", iv="0c78abc083b011e7")
    client.ping()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

from mootdx import __version__


def parse_requirements(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


with open('README.rst', encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst', encoding='utf-8') as history_file:
    history = history_file.read()

requirements = parse_requirements('requirements.txt')
test_requirements = requirements
test_requirements.append('pytest')

setup(
    name='mootdx',
    version=__version__,
    description="通达信数据读取接口.",
    long_description=readme + '\n\n' + history,
    author="bopo.wang",
    author_email='ibopo@126.com',
    url='https://github.com/bopo/mootdx',
    packages=find_packages(include=['mootdx', 'mootdx.*']),
    # include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='mootdx',
    entry_points={
        'console_scripts': [
            'mootdx = mootdx.__main__:execute',
        ]
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=requirements,
)



# -*- coding: utf-8 -*-
import logging
import os

from mootdx.consts import TYPE_GROUP
from mootdx.utils import get_stock_market
from pytdx.reader import (BlockReader, CustomerBlockReader, TdxDailyBarReader,
                          TdxExHqDailyBarReader, TdxLCMinBarReader)

logger = logging.getLogger(__name__)


# 股票市场
class Reader(object):
    @staticmethod
    def factory(market='std', **kwargs):
        '''
        Reader 工厂方法

        :param market:  std 标准市场, ext 扩展市场
        :param kwargs:
        :return:
        '''
        if market == 'ext':
            return ExtReader(**kwargs)
        elif market == 'std':
            return StdReader(**kwargs)


class ReaderBase(object):
    """股票市场"""

    # 默认通达信安装目录
    tdxdir = 'C:/new_tdx'

    def __init__(self, tdxdir=None):
        '''
        构造函数

        :param tdxdir: 通达信安装目录
        '''

        if os.path.isdir(tdxdir):
            self.tdxdir = tdxdir
        else:
            logger.error('tdxdir 目录不存在')

    def find_path(self, symbol=None, subdir='lday', ext=None):
        '''
        自动匹配文件路径，辅助函数

        :param symbol:
        :param subdir:
        :param ext:
        :return: pd.dataFrame or None
        '''
        market = get_stock_market(symbol, True) if len(symbol.split('#')) == 1 else 'ds'
        prefix = market if len(symbol.split('#')) == 1 else ''
        ext = ext if isinstance(ext, list) else [ext]

        for t in ext:
            vipdoc = 'vipdoc/{}/{}/{}{}.{}'.format(market, subdir, prefix, symbol, t)
            vipdoc = os.path.join(self.tdxdir, vipdoc)

            if os.path.exists(vipdoc):
                return vipdoc
            else:
                logger.error('未找到所需的文件: {}'.format(vipdoc))

        return None


class StdReader(ReaderBase):
    """股票市场"""

    def daily(self, symbol=None):
        '''
        获取日线数据

        :param symbol:
        :return: pd.dataFrame or None
        '''
        reader = TdxDailyBarReader()
        vipdoc = self.find_path(symbol=symbol, subdir='lday', ext='day')

        if vipdoc is not None:
            return reader.get_df(vipdoc)

        return None

    def minute(self, symbol=None, suffix=1):
        '''
        获取1,5分钟线

        :param suffix:
        :param symbol:
        :return: pd.dataFrame or None
        '''
        subdir = 'fzline' if str(suffix) == '5' else 'minline'
        suffix = ['lc5', '5'] if str(suffix) == '5' else ['lc1', '1']
        symbol = self.find_path(symbol, subdir=subdir, ext=suffix)
        reader = TdxLCMinBarReader()

        if symbol is not None:
            return reader.get_df(symbol)

        return None

    def fzline(self, symbol=None):
        return self.minute(symbol, suffix=5)

    def block(self, symbol='block', custom=False, group=False):
        '''
        获取板块数据
        参考: http://blog.sina.com.cn/s/blog_623d2d280102vt8y.html

        :param custom:
        :param symbol:
        :param group:
        :return: pd.dataFrame or None
        '''
        if custom:
            reader = CustomerBlockReader()
            vipdoc = os.path.join(self.tdxdir, 'T0002', 'blocknew', '{}'.format(symbol))
        else:
            reader = BlockReader()
            vipdoc = os.path.join(self.tdxdir, 'T0002', 'hq_cache', '{}.dat'.format(symbol))

        format = TYPE_GROUP if group else None

        if os.path.exists(vipdoc):
            return reader.get_df(vipdoc, format)
        else:
            logger.error('未找到所需的文件: {}'.format(vipdoc))

        return None


class ExtReader(ReaderBase):
    """扩展市场读取"""

    def daily(self, symbol=None):
        '''
        获取日线数据

        :return: pd.dataFrame or None
        '''
        reader = TdxExHqDailyBarReader()
        vipdoc = self.find_path(symbol=symbol, subdir='lday', ext='day')

        if symbol is not None:
            return reader.get_df(vipdoc)

        return None
